% ----------------------------------------------------------
% Giglio, Maggiori, and Stroebel, Econometrica,
% "No-Bubble Condition: Model-Free Tests in Housing Markets"
% ----------------------------------------------------------

% Code for bubble tests based largely on Matlab code from Phillips, Shi and Yu (2014)

clear
clc

close all

% Add directory with our functions
addpath Functions

set(0,'DefaultAxesFontName', 'Times New Roman')
set(0,'DefaultTextFontname', 'Times New Roman')

% UK and Singapore aggregate price and rent data
load data/gp.mat

% Disaggregated data for subregions
load data/estimatedseries.mat

% Note: prime central london (pcl), london, and UK subregionsregions are
% nominal, in logs, while uk_hpi and sg_hpi are real and in levels

% Make nominal series real
pcl_hpi=pcl-log(cpi);
london_hpi=london-log(cpi);
regions=regions-repmat(log(cpi),1,size(regions,2));
london_dates=yq;

clear london pcl cpi yq

% Max lags for lag order test.
maxlags_uk=5; %(years)
maxlags_sg=4*5; %(quarters)
maxlags_london=4*5; %(quarters), used also for all UK subregion analysis

% Rescale so that the log series start from 0 at the beginning of each
% sample
uk_hpi=uk_hpi/uk_hpi(1);
sg_hpi=sg_hpi/sg_hpi(1);
london_hpi=london_hpi-london_hpi(1);
pcl_hpi=pcl_hpi-pcl_hpi(1);
regions_hpi=regions-repmat(regions(1,:),size(regions,1),1);

% Find periods where we see both prices and rents in the UK and Singapore
f=find(~isnan(uk_rents));
nonmis_uk=f(1):length(uk_rents);
uk_rents=uk_rents/uk_rents(f(1))*uk_hpi(f(1));
f=find(~isnan(sg_rents));
sg_rents=sg_rents/sg_rents(f(1))*sg_hpi(f(1));
nonmis_sg=f(1):length(sg_rents);

% Lag selection
%--------------

% Use model with constant and trend since it is the most general one that
% nests all other models.
% Note that we need to also compute the case of zero lags, the first
% element of the BIC vector

% UK
clear BIC
for lags=0:maxlags_uk
    [~,~,BIC(lags+1)]=ADF_FL(log(uk_hpi),lags,2);
end
[~,nlags_uk_hpi]=min(BIC); nlags_uk_hpi=nlags_uk_hpi-1;

clear BIC
for lags=0:maxlags_uk
    [~,~,BIC(lags+1)]=ADF_FL(log(uk_rents(nonmis_uk)),lags,2);
end
[~,nlags_uk_rents]=min(BIC); nlags_uk_rents=nlags_uk_rents-1;

% SG
clear BIC
for lags=0:maxlags_sg
    [~,~,BIC(lags+1)]=ADF_FL(log(sg_hpi),lags,2);
end
[~,nlags_sg_hpi]=min(BIC); nlags_sg_hpi=nlags_sg_hpi-1;

clear BIC
for lags=0:maxlags_sg
    [~,~,BIC(lags+1)]=ADF_FL(log(sg_rents(nonmis_sg)),lags,2);
end
[~,nlags_sg_rents]=min(BIC); nlags_sg_rents=nlags_sg_rents-1;

% London
clear BIC
for lags=0:maxlags_london
    [~,~,BIC(lags+1)]=ADF_FL(london_hpi,lags,2);
end
[~,nlags_london_hpi]=min(BIC); nlags_london_hpi=nlags_london_hpi-1;

% PCL
clear BIC
for lags=0:maxlags_london
    [~,~,BIC(lags+1)]=ADF_FL(pcl_hpi,lags,2);
end
[~,nlags_pcl_hpi]=min(BIC); nlags_pcl_hpi=nlags_pcl_hpi-1;

% 100 regions
for i=1:100
    
    clear BIC
    for lags=0:maxlags_london
        [~,~,BIC(lags+1)]=ADF_FL(regions_hpi(:,i),lags,2);
    end
    [~,nlags_regions(i,1)]=min(BIC); nlags_regions(i,1)=nlags_regions(i,1)-1;
    
end


% Stationarity of log pd (restricted)
%------------------------------------

clear TT
alphalist=[0.1 0.05 0.01];
for alpha=1:3

    Y=log([uk_hpi(nonmis_uk) uk_rents(nonmis_uk)]);
    % Note: model 'ard' is stationary without a trend
    [h_pd(1),pValue_pd(1),stat_pd(1),cValue_pd(1),~]=adftest(Y(:,2)-Y(:,1),'model','ard','lags',nlags_uk_hpi,'test','t1','alpha',alphalist(alpha));

    Y=log([sg_hpi(nonmis_sg) sg_rents(nonmis_sg)]);
    [h_pd(2),pValue_pd(2),stat_pd(2),cValue_pd(2),~]=adftest(Y(:,2)-Y(:,1),'model','ard','lags',nlags_sg_hpi,'test','t1','alpha',alphalist(alpha));

    if alpha==1
        TT=[stat_pd' cValue_pd'];
    else
        TT=[TT cValue_pd'];
    end
    
end
TT=TT';



% Bubble dating and tests
%------------------------

% Change list for outputting so that the regions are in alphabetical order

regioncode_order=sort(regioncode);

clear TT
TT=[];
for i=1:104
    
    i
    
    % UK
    if i==1
        y=log(uk_hpi);
        T=length(y);
        lags=nlags_uk_hpi;
        dates=uk_dates;
        
    % SG    
    elseif i==2
        y=log(sg_hpi);
        T=length(y);
        lags=nlags_sg_hpi;
        dates=sg_dates; 
        
    % London    
    elseif i==3
        y=london_hpi;
        T=length(y);
        lags=nlags_london_hpi;
        dates=london_dates;
        
    % PCL    
    elseif i==4
        y=pcl_hpi;
        T=length(y);
        lags=nlags_pcl_hpi;
        dates=london_dates;
    
    % 100 regions
    elseif i>=5 && i<=104 
        % Find corresponding region
        fff=find(strcmp(regioncode_order{i-4},regioncode));
        y=regions_hpi(:,fff);
        T=length(y);
        lags=nlags_regions(fff);
        dates=london_dates;
   
    end
        
    % Find small-sample critical values
    C_ADF=criticalvalues_ADF(T);
    C_SADF=criticalvalues_SADF(T);
    C_GSADF=criticalvalues_GSADF(T);
    CR_BSADF=criticalvalues_BSADF(T);

    [ADF,SADF,GSADF,bsadfs]=runtests(y,lags);

    TT=[TT [ADF;C_ADF;SADF;C_SADF;GSADF;C_GSADF]];
    
    % First row is the date
    % second row is the statistic
    % Last three rows are the critical values    
    clear datingtest
    datingtest(1,:)=dates;
    datingtest(2,(T-length(bsadfs)+1:T))=bsadfs;
    datingtest(3:5,(T-length(bsadfs)+1:T))=CR_BSADF;
    
    % Rows of datingtest
    % 3: 10% level
    % 4: 5% level
    % 5: 1% level
    bb=datingtest(2,:)>datingtest(4,:); % Use 5% level
    
    % For dating tests on quarterly data, eliminate bubbles lasting only 1
    % quarter
    if i>1 
        
        for j=2:length(bb)-1
            if bb(j-1)==0 && bb(j+1)==0
                bb(j)=0;
            end
        end
        if bb(2)==0
            bb(1)=0;
        end
        if bb(end-1)==0;
            bb(end)=0;
        end
    end
    
    f=find(bb);
    
    % Show dates of bubble
    datingtest(1,f)
    
    % Save figure
    figure
    plot(y)
    hold on
    for ll=1:length(f)
        h=area([f(ll)-0.5 f(ll)+0.5],[max(y) max(y)]);
        set(h,'FaceColor',[0.8 0.8 0.8])
        set(h,'EdgeColor',[0.8 0.8 0.8])
    end
    plot(y,'b','linewidth',3) 
    
    ylabel('Price')
    axis tight
    plotaxis(datingtest(1,:),5)
    set(gcf, 'Color', 'w');
    set(gca,'FontSize',21)
    if i>=5 && i<=104
        title(['Postcode: ' regioncode_order{i-4}])
    end

    save2pdf(sprintf('figures/figtest_%1.0f.pdf',i));
    close
    
end

