*******************************************************************************************
*******************************************************************************************
* - "No-Bubble Condition: Model-Free Tests in Housing Markets"
*   Giglio, Maggiori, and Stroebel
*   Econometrica
*******************************************************************************************
*******************************************************************************************

*******************************************************************************************
* SINGAPORE DATA PROCESSING - PART 3
* - Clean Dataset, create dependent and control variables, etc.
*******************************************************************************************

use "Final Dataset.dta", clear

* Find first year of leasehold
gen length_ten_field     = length(tenure)
gen leasehold_first_year = substr(tenure, length_ten_field-3,.)
destring(leasehold_first_year), replace force

* Find how long leasehold
gen strpos_yrs      = strpos(tenure,"Yrs")
gen leasehold_years = substr(tenure,1,strpos_yrs-2)
destring(leasehold_years), replace force

drop strpos_yrs

* Drop if tenure is N.A. or larger than 999 years (recording errors) -- ~0.5% of transactions
drop if (leasehold_years > 999 & leasehold_years!=.) | (tenure=="N.A.")

* Find contract years
gen contract_year = substr(contractdate,length(contractdate) - 3,.)
destring(contract_year), force replace

* Find lease length remaining
gen lease_length_remain = leasehold_years - (contract_year - leasehold_first_year)

* Three different lease types
* FREEHOLD, EXTREMELY-LONG LEASEHOLD, SHORTER LEASEHOLD
* (There are no leaseholds issued with maturities between 103 years and 999 years, so everything above 103 is actually above 800 years)
gen     lease_type = 1 if lease_length_remain == .
replace lease_type = 2 if lease_length_remain <= 1000 & lease_length_remain > 103
replace lease_type = 3 if lease_length_remain <= 103

* Drop transactions involving multiple units (very few)
keep if noofunits==1
drop noofunits

bysort address: gen number_of_sales = _N
gen contract_month = substr(contractdate, length(contractdate)-7,3)

gen     month = 1  if contract_month == "JAN"
replace month = 2  if contract_month == "FEB"
replace month = 3  if contract_month == "MAR"
replace month = 4  if contract_month == "APR"
replace month = 5  if contract_month == "MAY"
replace month = 6  if contract_month == "JUN"
replace month = 7  if contract_month == "JUL"
replace month = 8  if contract_month == "AUG"
replace month = 9  if contract_month == "SEP"
replace month = 10 if contract_month == "OCT"
replace month = 11 if contract_month == "NOV"
replace month = 12 if contract_month == "DEC"

gen matlabsaledate = contract_year*100+month

gen date = "01\" + string(month) + "-" + string(contract_year)
gen date1 = date(date,"DMY")
format date1 %td

gen year = year(date1)

* Remove duplicates, drop variables that are not needed
duplicates drop address contractdate transactedprice areasqm, force
drop contractdate _merge developer length_ten_field contract_month date

*************************************
* Now Add Control Vars
*************************************

* Other transaction-level controls 
gen new = (typeofsale == "New Sale")

* Lease-length indicator
cap drop ll_remain3 
gen      ll_remain3 = .
replace  ll_remain3 = 0  if lease_type == 1
replace  ll_remain3 = 1  if lease_length_remain <  70  & lease_length_remain >  50 & ll_remain3 == .
replace  ll_remain3 = 2  if lease_length_remain <  85  & lease_length_remain >= 70
replace  ll_remain3 = 3  if lease_length_remain <  90  & lease_length_remain >= 85
replace  ll_remain3 = 4  if lease_length_remain <  95  & lease_length_remain >= 90
replace  ll_remain3 = 5  if lease_length_remain <= 103 & lease_length_remain >= 95
replace  ll_remain3 = 6  if lease_length_remain >  800 & lease_length_remain != .    
 
* Create indicators for remaining lease length for regression
tab ll_remain3, gen(llr)

* Additional control variables
egen prop_type   = group(propertytype)
egen size_group2 = cut(areasqm), group(40)
gen incomplete   = (completiondate == "Uncompleted")
destring(completiondate), replace force
gen nocompletioninfo=(completiondate==.)
 
* Age
gen     age = contract_year - completiondate
replace age = 0 if incomplete == 1
replace age = 0 if age < 0
replace age = 0 if nocompletioninfo==1

* Contract Dates
gen contract_quarter = qofd(date1)
gen contract_month   = mofd(date1)

* Postal Code FE
gen round_postal3 = round(postalcode/10)

* How many units in project (Indicator for Groups)
gen      total_units_group = 1 if totalunitsinproject <= 10
replace  total_units_group = 2 if totalunitsinproject >  10   & totalunitsinproject <= 50
replace  total_units_group = 3 if totalunitsinproject >  50   & totalunitsinproject <= 100
replace  total_units_group = 4 if totalunitsinproject >  100  & totalunitsinproject <= 200
replace  total_units_group = 5 if totalunitsinproject >  200  & totalunitsinproject <= 500
replace  total_units_group = 6 if totalunitsinproject >  500  & totalunitsinproject <= 1000
replace  total_units_group = 7 if totalunitsinproject >  1000 & totalunitsinproject != .
replace  total_units_group = 0 if totalunitsinproject == .

* FIXED EFFECTS FOR REGRESSION
egen time_loc_type1 = group(typeofarea round_postal3 contract_year    prop_type)
egen time_loc_type2 = group(typeofarea round_postal3 contract_quarter prop_type)
egen time_loc_type3 = group(typeofarea round_postal3 contract_month   prop_type)

**************************************
* Left-hand Side Variables
**************************************

gen    log_price = log(transactedprice)
winsor log_price, p(0.05) gen(log_price_w05)

saveold "cleaned_data.dta", replace
