*******************************************************************************************
*******************************************************************************************
* - "No-Bubble Condition: Model-Free Tests in Housing Markets"
*   Giglio, Maggiori, and Stroebel
*   Econometrica
*******************************************************************************************
*******************************************************************************************

**************************************************************************************************
* Some additional data cleaning of lease data
* - Much of this is based on small irregularities detected in the raw data from the Land Registry,
*   that were explained through phone and email conversations with Alison Draper
* - Many of these relate to the infrequent and incomplete recording of lease extensions, and thus
*   do not affect either freehold contracts or the extremely-long leaseholds that are the focus of
*   the paper "No-Bubble Condition: Model-Free Tests in Housing Markets"	.
**************************************************************************************************

cap use "Data\MergedDataRaw.dta", clear

* Generate indicators for unique properties
egen house_id_prop = group(postcode street_number flat_number street prop_type), missing
egen house_id      = group(postcode street_number flat_number street), missing

* Leasehold indicator
gen leasehold = (duration == "L")
gen year_from = year(date_from)

* Drop leaseholds claiming to have originated prior to 1200
drop if year_from < 1200 // * About 0.0005% of transactions

* Drop if we have a leasehold transaction for which the land registry data does not provide lease length information
* This is about 2% of all lease transactions in the Price Paid Dataset
drop if duration == "L" & (number_years == . | date_from == .)

* Drop leaseholds where at transaction data we would have negative years remaining
* About 0.001% of transactions
cap drop years_remaining
gen      years_remaining = number_years - round((date_trans - date_from) / 365)
replace  years_remaining = . if years_remaining < 0

*******************************************************************************************************
* REMOVE HEDONICS WHEN THERE IS A "NEW" PROPERTY INDICATOR FOR A LATER TRANSACTION AT SAME ADDRESS
* - This is because Rightmove/Zoopla will have hedonics for the property as it looks
*   today - we don't want to assign those characteristics to earlier transactions of
*   a different property
*******************************************************************************************************

gen     new_prop = (old_new == "Y")
gsort   house_id -date_trans
by      house_id: gen later_new = sum(new_prop)
replace later_new = 0 if new_prop == 1

gen miss_hed = 1 if later_new > 0

********************************************************************************************
* Property type changes
* - Create a variable "prop_type_golden" that measures whether we are concerned about the 
*   quality of the "property-type"
******************************************************************************************** 

* PTID gives, per address, the number of different property types selling there
* 98.8% of transactions are for properties at an address where the property type does not change
bysort house_id (date_trans): gen ptid = sum(prop_type!=prop_type[_n-1])
egen   ptidrankmax = max(ptid), by(house_id)
order  ptid ptidrankmax prop_type house_id leasehold

* All properties that have only property type transacting at the addess already have prop_type_golden to be set equal to 1
gen prop_type_golden = (ptidrankmax==1)

* If it's all freeholds, keep them all but put hedonics to missing. Unlike for leaseholds, we are not worried about registration of lease changes as property type changes.
* We remove hedonics because we are unsure which property types the Rightmove data corresponds to.
egen allfh=max(leasehold), by(house_id)
replace prop_type_golden=2 if allfh==0 & ptidrankmax>1
bysort house_id (date_trans): replace miss_hed=1 if _n<_N & prop_type_golden==2 & ptidrankmax>1

* Remove hedonics if so determined
foreach var in bedrooms bathrooms conservatory living property_style built condition_ind heating_ind parking_ind floor_area {
	replace `var' = 99 if miss_hed == 1
}
	
********************************************************************
* CREATE SOME MORE VARIABLES THAT WE WILL USE LATER
********************************************************************

gen  log_price     = log(price)
gen  month_of_sale = mofd(date_trans)
gen  year          = year(date_trans)
gen  quarter       = qofd(date_trans)
egen proptype      = group(prop_type)
gen  is_flat       = (prop_type=="F")

* Drop some duplicates
duplicates drop house_id date_trans price leasehold is_flat date_from number_years new_prop postcode, force

winsor log_price, gen(log_price_win) p(0.05)

* Generate control variable dummies
cap drop floor_group
egen     floor_group = cut(floor_area), group(50)
replace  floor_group = 99 if floor_group == .

gen      age = year - built if built != 99
replace  age = . if age < 0
cap drop age_group
egen     age_group = cut(age), group(50) 
replace  age_group = 99 if age_group == .

* Drop duplicates in case any remain
duplicates drop house_id_prop log_price date_trans leasehold date_from number_years floor_area bedrooms bathrooms, force

*********************************************		 
* CREATE NEW VARIABLES		 		 
*********************************************

gen     tom = date_trans - list_start_dt
replace tom = . if tom < 0
replace tom = . if tom > 500

gen log_tom = log(tom)

* Remaining Lease Length group indicators for houses
cap drop years_group_new_house      
gen      years_group_new_house  = 0 if leasehold == 0
replace  years_group_new_house  = 1 if (years_remaining <  100) & (years_remaining >= 80)  & (years_remaining != .) & leasehold == 1
replace  years_group_new_house  = 2 if (years_remaining <  125) & (years_remaining >= 100) & (years_remaining != .) & leasehold == 1
replace  years_group_new_house  = 3 if (years_remaining <= 200) & (years_remaining >= 125) & (years_remaining != .) & leasehold == 1
replace  years_group_new_house  = 4 if (years_remaining >= 700) & (years_remaining < 1000) & (years_remaining != .) & leasehold == 1

cap drop yg_house*
tab years_group_new_house, gen(yg_house)

* Remaining Lease Length group indicators for flats
cap drop years_group_new_flat      
gen      years_group_new_flat  = 0 if leasehold == 0
replace  years_group_new_flat  = 1 if (years_remaining <  100) & (years_remaining >= 80)  & (years_remaining != .) & leasehold == 1
replace  years_group_new_flat  = 2 if (years_remaining <  125) & (years_remaining >= 100) & (years_remaining != .) & leasehold == 1
replace  years_group_new_flat  = 3 if (years_remaining <  150) & (years_remaining >= 125) & (years_remaining != .) & leasehold == 1
replace  years_group_new_flat  = 4 if (years_remaining <= 300) & (years_remaining >= 150) & (years_remaining != .) & leasehold == 1
replace  years_group_new_flat  = 5 if (years_remaining >= 800) & (years_remaining < 1000) & (years_remaining != .) & leasehold == 1

cap drop yg_flat*
tab years_group_new_flat, gen(yg_flat)

************************************
* Construct Fixed Effects
************************************

gen pos_empty   = strpos(postcode," ")
gen post_0      = substr(postcode,1,pos_empty)
gen post_2      = substr(postcode,1,pos_empty+2)
replace post_0  = trim(post_0)
drop pos_empty

* Fixed effects for regressions
egen fe_12   = group(post_0  year    proptype)
egen fe_12_q = group(post_0  quarter proptype)
egen fe_12_m = group(post_0  month   proptype)

***************************************************************************
* INDICATOR VARIABLES FOR WHAT TRANSACTIONS TO INCLUDE
***************************************************************************

egen address  = group(street_number street postcode)
egen lfmaxtot = max(leasehold), by(address)
egen lfmintot = min(leasehold), by(address)

gen has_within_variation_add = (lfmaxtot != lfmintot)

* Only those leasehold transactions that we assign to one of the dummy variables for remaining lease length
gen      clean_ll = 1 
replace  clean_ll = 0 if (leasehold == 1 & years_group_new_house == .)

* Only those transactions where (1) we don't have variation within property of leasehold/freehold -- those are hard to interpret, and likely often driven by recording errors --
* and (ii) those where we don't have changes in property type within address (see above)
gen      clean1 = 0
replace  clean1 = 1 if has_within_variation_add == 0 & prop_type_golden > 0

egen characteristics_fe  = group(post_0  proptype) 

***************************************************
* Define bubble geographies
***************************************************

gen mayfair     = 0
replace mayfair = 1 if substr(postcode,1,2) == "W1"

gen knightsbridge     = 0
replace knightsbridge = 1 if substr(postcode,1,3) == "SW7" 

gen belgravia     = 0
replace belgravia = 1 if substr(postcode,1,3) == "SW1" 

gen chelsea     = 0
replace chelsea = 1 if substr(postcode,1,3) == "SW3" 

gen kensington     = 0
replace kensington = 1 if substr(postcode,1,2) == "W8"  | substr(postcode,1,3) == "SW5"  | substr(postcode,1,3) == "W14" 

gen pcl     = 0
replace pcl = 1 if mayfair == 1 | knightsbridge == 1 | belgravia == 1 | chelsea == 1 | kensington == 1 

compress
cap drop _merge

save "Data\bubble.dta", replace
