*******************************************************************************************
*******************************************************************************************
* - "No-Bubble Condition: Model-Free Tests in Housing Markets"
*   Giglio, Maggiori, and Stroebel
*   Econometrica
*******************************************************************************************
*******************************************************************************************

*******************************************************************************************
*
* RENTAL PRICE ANALYSIS FOR SINGAPORE - PART 6
* 
* It is not possible to merge the iProperty data at the property level to the 
* transaction data. For "for-rent" properties, we don't know what lease type the
* property is. So in this code we exploit that we have essentially no variation 
* in lease type within each 6-digit postcode. We can therefore assign a leasetype 
* to each property based on the exact postcode the property is located in.
*
*******************************************************************************************

use "cleaned_data.dta", clear

bysort postalcode: egen minll = min(lease_type)
bysort postalcode: egen maxll = max(lease_type)

egen   title = group(typeofarea)
bysort postalcode: egen mintt = min(title)
bysort postalcode: egen maxtt = max(title)

* Only keep those postcodes (vast majority) with unique lease type and title type in 6-digit postal code
* --> For those we can then attach the title type and lease type to the for-rent data
*     based in the 6-digit postal code
keep if minll == maxll
keep if mintt == maxtt

collapse (first) title lease_type, by(postalcode)

sort     postalcode
tempfile contract_type
save     `contract_type'

*************************************************************
* Get rental listings data from iProperty
*************************************************************

use "Listings\iPropertySG-listings.dta", clear

* Only keep the for-rent sample
keep if for_rent == 1

* Define listing-time effect
gen quarter = qofd(date_new)
gen month   = mofd(date_new)
gen year    = year(date_new)

duplicates drop postalcode price size quarter, force

destring(postalcode), replace force

* Drop those we cannot locate, then merge in lease type
drop if postalcode == .
drop if postalcode == 0

sort  postalcode
merge postalcode using `contract_type'

keep if _merge == 3

* Create more variables
replace protype  = trim(protype)

* Generate Fixed effects that mirror main regression
gen  pc3 = round(postalcode/10)
egen fe1 = group(year    pc3 protype title)
egen fe2 = group(quarter pc3 protype title)
egen fe3 = group(month   pc3 protype title)

*********************************************
* Data Cleaning
*********************************************

replace size    = . if size == 0
egen size_group = cut(size), group(40)

* Deal with property age
replace age = trim(age)
replace age = subinstr(age,"yr","",.)
replace age = subinstr(age,"y","",.)

destring(age), replace force
replace age = . if age < 0
replace age = round(age)

gen     aa = age
replace aa = 99 if aa == .

* Construct dependent variable
gen  log_price = log(price)

*************************
* Now regressions
*************************

* Table IV, Column 1
areg log_price i.aa i.size_group i.lease_type if price > 10000 & price < 300000, absorb(fe1) cluster(pc3) 

* Table IV, Column 2
areg log_price i.aa i.size_group i.lease_type if price > 10000 & price < 300000, absorb(fe3) cluster(pc3) 
